@@ -1,8 +1,8 @@
 # COPYRIGHT (C) 2020-2022 Nicotine+ Contributors
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
-# COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
-# COPYRIGHT (C) 2007 Gallows <g4ll0ws@gmail.com>
-# COPYRIGHT (C) 2006-2009 Daelstorm <daelstorm@gmail.com>
+# COPYRIGHT (C) 2008-2011 quinox <quinox@users.sf.net>
+# COPYRIGHT (C) 2007 gallows <g4ll0ws@gmail.com>
+# COPYRIGHT (C) 2006-2009 daelstorm <daelstorm@gmail.com>
 # COPYRIGHT (C) 2003-2004 Hyriand <hyriand@thegraveyard.org>
 #
 # GNU GENERAL PUBLIC LICENSE
@@ -30,7 +30,7 @@ from gi.repository import GLib
 
 from pynicotine import slskmessages
 from pynicotine.config import config
-from pynicotine.gtkgui.application import GTK_API_VERSION
+from pynicotine.gtkgui.popovers.privatechatcommands import PrivateChatCommands
 from pynicotine.gtkgui.widgets.iconnotebook import IconNotebook
 from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
 from pynicotine.gtkgui.widgets.popupmenu import UserPopupMenu
@@ -62,13 +62,7 @@ class PrivateChats(IconNotebook):
         )
 
         self.completion = ChatCompletion()
-
-        self.command_help = UserInterface("ui/popovers/privatechatcommands.ui")
-        self.command_help.container, self.command_help.popover = self.command_help.widgets
-
-        if GTK_API_VERSION >= 4:
-            # Scroll to the focused widget
-            self.command_help.container.get_child().set_scroll_to_focus(True)
+        self.command_help = None
 
     def on_switch_chat(self, _notebook, page, _page_num):
 
@@ -79,11 +73,12 @@ class PrivateChats(IconNotebook):
             if tab.container != page:
                 continue
 
-            GLib.idle_add(lambda tab: tab.chat_entry.grab_focus() == -1, tab)
-
             self.completion.set_entry(tab.chat_entry)
             tab.set_completion_list(self.core.privatechats.completion_list[:])
 
+            if self.command_help is None:
+                self.command_help = PrivateChatCommands(self.frame.window)
+
             self.command_help.popover.unparent()
             tab.help_button.set_popover(self.command_help.popover)
 
@@ -133,7 +128,8 @@ class PrivateChats(IconNotebook):
 
         if user not in self.pages:
             self.pages[user] = page = PrivateChat(self, user)
-            self.append_page(page.container, user, page.on_close, user=user)
+            self.append_page(page.container, user, focus_callback=page.on_focus,
+                             close_callback=page.on_close, user=user)
             page.set_label(self.get_tab_label_inner(page.container))
 
         if switch_page and self.get_current_page() != self.pages[user].container:
@@ -248,8 +244,8 @@ class PrivateChat(UserInterface):
                 ("#" + _("_Close Tab"), self.on_close)
             )
 
-        popup = PopupMenu(self.frame, self.chat_view.textview, self.on_popup_menu_chat)
-        popup.add_items(
+        self.popup_menu = PopupMenu(self.frame, self.chat_view.textview, self.on_popup_menu_chat)
+        self.popup_menu.add_items(
             ("#" + _("Find…"), self.on_find_chat_log),
             ("", None),
             ("#" + _("Copy"), self.chat_view.on_copy_text),
@@ -316,9 +312,13 @@ class PrivateChat(UserInterface):
         self.update_local_username_tag(status=UserStatus.OFFLINE)
 
     def clear(self):
+
         self.chat_view.clear()
         self.frame.notifications.clear("private", self.user)
 
+        for menu in (self.popup_menu_user_chat, self.popup_menu_user_tab, self.popup_menu):
+            menu.clear()
+
     def set_label(self, label):
         self.popup_menu_user_tab.set_parent(label)
 
@@ -345,7 +345,6 @@ class PrivateChat(UserInterface):
 
         if response_id == 2:
             delete_log(config.sections["logging"]["privatelogsdir"], self.user)
-            self.chats.history.remove_user(self.user)
             self.chat_view.clear()
 
     def on_delete_chat_log(self, *_args):
@@ -472,11 +471,7 @@ class PrivateChat(UserInterface):
         color = get_user_status_color(self.status)
         self.tag_username = self.chat_view.create_tag(color, callback=self.user_name_event, username=self.user)
 
-        if not self.core.logged_in:
-            color = "useroffline"
-        else:
-            color = "useraway" if self.core.away else "useronline"
-
+        color = get_user_status_color(self.core.user_status)
         my_username = config.sections["server"]["login"]
         self.tag_my_username = self.chat_view.create_tag(color, callback=self.user_name_event, username=my_username)
 
@@ -500,6 +495,9 @@ class PrivateChat(UserInterface):
                     self.tag_username, self.tag_my_username):
             self.chat_view.update_tag(tag)
 
+    def on_focus(self, *_args):
+        self.chat_entry.grab_focus()
+
     def on_close(self, *_args):
         self.core.privatechats.remove_user(self.user)
 
