@@ -1,7 +1,7 @@
 # COPYRIGHT (C) 2020-2022 Nicotine+ Contributors
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
-# COPYRIGHT (C) 2008-2010 Quinox <quinox@users.sf.net>
-# COPYRIGHT (C) 2006-2009 Daelstorm <daelstorm@gmail.com>
+# COPYRIGHT (C) 2008-2010 quinox <quinox@users.sf.net>
+# COPYRIGHT (C) 2006-2009 daelstorm <daelstorm@gmail.com>
 # COPYRIGHT (C) 2003-2004 Hyriand <hyriand@thegraveyard.org>
 #
 # GNU GENERAL PUBLIC LICENSE
@@ -54,20 +54,9 @@ class UserInfos(IconNotebook):
         super().__init__(
             frame, core,
             widget=frame.userinfo_notebook,
-            parent_page=frame.userinfo_page,
-            switch_page_callback=self.on_switch_info_page
+            parent_page=frame.userinfo_page
         )
 
-    def on_switch_info_page(self, _notebook, page, _page_num):
-
-        if self.frame.current_page_id != self.frame.userinfo_page.id:
-            return
-
-        for tab in self.pages.values():
-            if tab.container == page:
-                GLib.idle_add(lambda tab: tab.description_view.textview.grab_focus() == -1, tab)
-                break
-
     def on_get_user_info(self, *_args):
 
         username = self.frame.userinfo_entry.get_text().strip()
@@ -82,7 +71,8 @@ class UserInfos(IconNotebook):
 
         if user not in self.pages:
             self.pages[user] = page = UserInfo(self, user)
-            self.append_page(page.container, user, page.on_close, user=user)
+            self.append_page(page.container, user, focus_callback=page.on_focus,
+                             close_callback=page.on_close, user=user)
             page.set_label(self.get_tab_label_inner(page.container))
 
         if switch_page:
@@ -104,9 +94,13 @@ class UserInfos(IconNotebook):
         if user in self.pages:
             self.pages[user].show_connection_error()
 
-    def message_progress(self, msg):
+    def peer_message_progress(self, msg):
         if msg.user in self.pages:
-            self.pages[msg.user].message_progress(msg)
+            self.pages[msg.user].peer_message_progress(msg)
+
+    def peer_connection_closed(self, msg):
+        if msg.user in self.pages:
+            self.pages[msg.user].peer_connection_closed()
 
     def get_user_stats(self, msg):
         if msg.user in self.pages:
@@ -159,6 +153,8 @@ class UserInfo(UserInterface):
             self.placeholder_picture,
             self.progress_bar,
             self.queued_uploads_label,
+            self.refresh_button,
+            self.retry_button,
             self.shared_files_label,
             self.shared_folders_label,
             self.upload_slots_label,
@@ -170,7 +166,7 @@ class UserInfo(UserInterface):
         self.frame = userinfos.frame
         self.core = userinfos.core
 
-        self.info_bar = InfoBar(self.info_bar, Gtk.MessageType.INFO)
+        self.info_bar = InfoBar(self.info_bar, button=self.retry_button)
         self.description_view = TextView(self.description_view)
         self.user_label.set_text(user)
 
@@ -194,6 +190,7 @@ class UserInfo(UserInterface):
         self.picture_data_original = self.picture_data_scaled = None
         self.zoom_factor = 5
         self.actual_zoom = 0
+        self.indeterminate_progress = True
 
         # Set up likes list
         self.likes_store = Gtk.ListStore(str)
@@ -222,9 +219,9 @@ class UserInfo(UserInterface):
         self.dislikes_list_view.set_model(self.dislikes_store)
 
         # Popup menus
-        self.user_popup = popup = UserPopupMenu(self.frame, None, self.on_tab_popup)
-        popup.setup_user_menu(user, page="userinfo")
-        popup.add_items(
+        self.user_popup_menu = UserPopupMenu(self.frame, None, self.on_tab_popup)
+        self.user_popup_menu.setup_user_menu(user, page="userinfo")
+        self.user_popup_menu.add_items(
             ("", None),
             ("#" + _("Close All Tabs…"), self.on_close_all_tabs),
             ("#" + _("_Close Tab"), self.on_close)
@@ -236,14 +233,14 @@ class UserInfo(UserInterface):
                     ("", None),
                     ("#" + _("_Search for Item"), self.on_interest_recommend_search, popup))
 
-        popup = PopupMenu(self.frame, self.likes_list_view, self.on_popup_interest_menu)
-        popup.add_items(*get_interest_items(popup))
+        self.likes_popup_menu = PopupMenu(self.frame, self.likes_list_view, self.on_popup_interest_menu)
+        self.likes_popup_menu.add_items(*get_interest_items(self.likes_popup_menu))
 
-        popup = PopupMenu(self.frame, self.dislikes_list_view, self.on_popup_interest_menu)
-        popup.add_items(*get_interest_items(popup))
+        self.dislikes_popup_menu = PopupMenu(self.frame, self.dislikes_list_view, self.on_popup_interest_menu)
+        self.dislikes_popup_menu.add_items(*get_interest_items(self.dislikes_popup_menu))
 
-        popup = PopupMenu(self.frame, self.picture_view)
-        popup.add_items(
+        self.picture_popup_menu = PopupMenu(self.frame, self.picture_view)
+        self.picture_popup_menu.add_items(
             ("#" + _("Zoom 1:1"), self.make_zoom_normal),
             ("#" + _("Zoom In"), self.make_zoom_in),
             ("#" + _("Zoom Out"), self.make_zoom_out),
@@ -252,6 +249,7 @@ class UserInfo(UserInterface):
         )
 
         self.update_visuals()
+        self.set_in_progress()
 
     def clear(self):
 
@@ -260,8 +258,12 @@ class UserInfo(UserInterface):
         self.dislikes_store.clear()
         self.load_picture(None)
 
+        for menu in (self.user_popup_menu, self.likes_popup_menu, self.dislikes_popup_menu,
+                     self.likes_list_view.column_menu, self.dislikes_list_view.column_menu, self.picture_popup_menu):
+            menu.clear()
+
     def set_label(self, label):
-        self.user_popup.set_parent(label)
+        self.user_popup_menu.set_parent(label)
 
     def save_columns(self):
         # Unused
@@ -362,16 +364,43 @@ class UserInfo(UserInterface):
 
         self.info_bar.show_message(
             _("Unable to request information from user. Either you both have a closed listening "
-              "port, the user is offline, or there's a temporary connectivity issue.")
+              "port, the user is offline, or there's a temporary connectivity issue."),
+            message_type=Gtk.MessageType.ERROR
         )
 
         self.set_finished()
 
     def set_finished(self):
+
+        self.indeterminate_progress = False
+
         self.userinfos.request_tab_hilite(self.container)
         self.progress_bar.set_fraction(1.0)
 
-    def message_progress(self, msg):
+        self.refresh_button.set_sensitive(True)
+
+    def pulse_progress(self, repeat=True):
+
+        if not self.indeterminate_progress:
+            return False
+
+        self.progress_bar.pulse()
+        return repeat
+
+    def set_in_progress(self):
+
+        self.indeterminate_progress = True
+
+        self.progress_bar.pulse()
+        GLib.timeout_add(320, self.pulse_progress, False)
+        GLib.timeout_add(1000, self.pulse_progress)
+
+        self.info_bar.set_visible(False)
+        self.refresh_button.set_sensitive(False)
+
+    def peer_message_progress(self, msg):
+
+        self.indeterminate_progress = False
 
         if msg.total == 0 or msg.position == 0:
             fraction = 0.0
@@ -382,6 +411,10 @@ class UserInfo(UserInterface):
 
         self.progress_bar.set_fraction(fraction)
 
+    def peer_connection_closed(self):
+        if not self.refresh_button.get_sensitive():
+            self.show_connection_error()
+
     """ Network Messages """
 
     def user_info_reply(self, msg):
@@ -413,11 +446,11 @@ class UserInfo(UserInterface):
 
     def set_user_country(self, country_code):
 
-        if country_code:
-            country = GeoIP.country_code_to_name(country_code)
-            country_text = "%s (%s)" % (country, country_code)
-        else:
-            country_text = _("Unknown")
+        if not country_code:
+            return
+
+        country = GeoIP.country_code_to_name(country_code)
+        country_text = "%s (%s)" % (country, country_code)
 
         self.country_label.set_text(country_text)
 
@@ -440,7 +473,7 @@ class UserInfo(UserInterface):
     """ Callbacks """
 
     def on_tab_popup(self, *_args):
-        self.user_popup.toggle_user_items()
+        self.user_popup_menu.toggle_user_items()
 
     def on_popup_interest_menu(self, menu, widget):
 
@@ -524,12 +557,12 @@ class UserInfo(UserInterface):
         return True
 
     def on_refresh(self, *_args):
-
-        self.info_bar.set_visible(False)
-        self.progress_bar.set_fraction(0.0)
-
+        self.set_in_progress()
         self.core.userinfo.request_user_info(self.user)
 
+    def on_focus(self, *_args):
+        self.description_view.textview.grab_focus()
+
     def on_close(self, *_args):
         self.core.userinfo.remove_user(self.user)
 
