@@ -2,10 +2,10 @@
 # COPYRIGHT (C) 2020-2022 Mathias <mail@mathias.is>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
-# COPYRIGHT (C) 2013 eL_vErDe <gandalf@le-vert.net>
-# COPYRIGHT (C) 2008-2012 Quinox <quinox@users.sf.net>
-# COPYRIGHT (C) 2009 Hedonist <ak@sensi.org>
-# COPYRIGHT (C) 2006-2009 Daelstorm <daelstorm@gmail.com>
+# COPYRIGHT (C) 2013 eLvErDe <gandalf@le-vert.net>
+# COPYRIGHT (C) 2008-2012 quinox <quinox@users.sf.net>
+# COPYRIGHT (C) 2009 hedonist <ak@sensi.org>
+# COPYRIGHT (C) 2006-2009 daelstorm <daelstorm@gmail.com>
 # COPYRIGHT (C) 2003-2004 Hyriand <hyriand@thegraveyard.org>
 # COPYRIGHT (C) 2001-2003 Alexander Kanavin
 #
@@ -53,7 +53,6 @@ from pynicotine.slskmessages import LoginFailure
 from pynicotine.slskmessages import UserStatus
 from pynicotine.transfers import Statistics
 from pynicotine.transfers import Transfers
-from pynicotine.upnp import UPnP
 from pynicotine.userbrowse import UserBrowse
 from pynicotine.userinfo import UserInfo
 from pynicotine.userlist import UserList
@@ -81,7 +80,6 @@ class NicotineCore:
         self.pluginhandler = None
         self.now_playing = None
         self.protothread = None
-        self.upnp = None
         self.geoip = None
         self.notifications = None
 
@@ -93,8 +91,7 @@ class NicotineCore:
         self.port = port
 
         self.shutdown = False
-        self.away = False
-        self.logged_in = False
+        self.user_status = UserStatus.OFFLINE
         self.login_username = None  # Only present while logged in
         self.user_ip_address = None
         self.privileges_left = None
@@ -119,7 +116,14 @@ class NicotineCore:
         log.add_debug("Using %(program)s executable: %(exe)s", {"program": config.application_name, "exe": script_dir})
         log.add(_("Loading %(program)s %(version)s"), {"program": config.application_name, "version": config.version})
 
-        self.geoip = GeoIP(os.path.join(script_dir, "geoip/ipcountrydb.bin"))
+        self.protothread = slskproto.SlskProtoThread(
+            core_callback=self.network_callback, queue=self.queue, bindip=self.bindip, port=self.port,
+            interface=config.sections["server"]["interface"],
+            port_range=config.sections["server"]["portrange"]
+        )
+        self.protothread.start()
+
+        self.geoip = GeoIP()
         self.notifications = Notifications(config, ui_callback)
         self.network_filter = NetworkFilter(self, config, self.queue, self.geoip)
         self.now_playing = NowPlaying(config)
@@ -138,11 +142,6 @@ class NicotineCore:
         self.transfers.init_transfers()
         self.privatechats.load_users()
 
-        port_range = config.sections["server"]["portrange"]
-        interface = config.sections["server"]["interface"]
-        self.protothread = slskproto.SlskProtoThread(
-            self.network_callback, self.queue, self.bindip, interface, self.port, port_range)
-        self.upnp = UPnP(self, config)
         self.pluginhandler = PluginHandler(self, config)
 
         # Callback handlers for messages
@@ -167,7 +166,7 @@ class NicotineCore:
             slskmessages.PierceFireWall: self.dummy_message,
             slskmessages.ConnectToPeer: self.connect_to_peer,
             slskmessages.CantConnectToPeer: self.dummy_message,
-            slskmessages.MessageProgress: self.message_progress,
+            slskmessages.PeerMessageProgress: self.peer_message_progress,
             slskmessages.SharedFileList: self.userbrowse.shared_file_list,
             slskmessages.GetSharedFileList: self.shares.get_shared_file_list,
             slskmessages.FileSearchRequest: self.dummy_message,
@@ -182,6 +181,7 @@ class NicotineCore:
             slskmessages.UploadFile: self.transfers.file_upload,
             slskmessages.FileDownloadInit: self.transfers.file_download_init,
             slskmessages.FileUploadInit: self.transfers.file_upload_init,
+            slskmessages.FileOffset: self.dummy_message,
             slskmessages.TransferRequest: self.transfers.transfer_request,
             slskmessages.TransferResponse: self.transfers.transfer_response,
             slskmessages.QueueUpload: self.transfers.queue_upload,
@@ -190,8 +190,9 @@ class NicotineCore:
             slskmessages.PlaceInQueue: self.transfers.place_in_queue,
             slskmessages.DownloadFileError: self.transfers.download_file_error,
             slskmessages.UploadFileError: self.transfers.upload_file_error,
-            slskmessages.DownloadConnClose: self.transfers.download_conn_close,
-            slskmessages.UploadConnClose: self.transfers.upload_conn_close,
+            slskmessages.DownloadConnectionClosed: self.transfers.download_connection_closed,
+            slskmessages.UploadConnectionClosed: self.transfers.upload_connection_closed,
+            slskmessages.PeerConnectionClosed: self.peer_connection_closed,
             slskmessages.FolderContentsResponse: self.transfers.folder_contents_response,
             slskmessages.FolderContentsRequest: self.shares.folder_contents_request,
             slskmessages.RoomList: self.chatrooms.room_list,
@@ -274,11 +275,8 @@ class NicotineCore:
 
         # Shut down networking thread
         if self.protothread:
-            if not self.protothread.server_disconnected:
-                self.protothread.manual_server_disconnect = True
-                self.server_disconnect()
-
             self.protothread.abort()
+            self.server_disconnect()
 
         # Save download/upload list to file
         if self.transfers:
@@ -349,18 +347,18 @@ class NicotineCore:
     def disconnect(self):
         self.queue.append(slskmessages.ServerDisconnect())
 
-    def send_message_to_peer(self, user, message, address=None):
+    def send_message_to_peer(self, user, message):
         """ Sends message to a peer. Used when we know the username of a peer,
         but don't have/know an active connection. """
 
-        self.queue.append(slskmessages.SendNetworkMessage(user, message, address))
+        self.queue.append(slskmessages.SendNetworkMessage(user, message))
 
     def set_away_mode(self, is_away, save_state=False):
 
         if save_state:
             config.sections["server"]["away"] = is_away
 
-        self.away = is_away
+        self.user_status = UserStatus.AWAY if is_away else UserStatus.ONLINE
         self.request_set_status(is_away and 1 or 2)
 
         # Reset away message users
@@ -387,7 +385,7 @@ class NicotineCore:
         """ Retrieve a user's country code if previously cached, otherwise request
         user's IP address to determine country """
 
-        if not self.logged_in:
+        if self.user_status == UserStatus.OFFLINE:
             return None
 
         user_address = self.protothread.user_addresses.get(user)
@@ -406,7 +404,7 @@ class NicotineCore:
         """ Tell the server we want to be notified of status/stat updates
         for a user """
 
-        if not self.logged_in:
+        if self.user_status == UserStatus.OFFLINE:
             return
 
         if not force_update and user in self.watched_users:
@@ -446,10 +444,10 @@ class NicotineCore:
 
         for i in msg.msgs:
             if i.__class__ in (slskmessages.TransferRequest, slskmessages.FileUploadInit):
-                self.transfers.get_cant_connect_upload(i.token)
+                self.transfers.get_cant_connect_upload(msg.user, i.token, msg.offline)
 
             elif i.__class__ is slskmessages.QueueUpload:
-                self.transfers.get_cant_connect_queue_file(msg.user, i.file)
+                self.transfers.get_cant_connect_queue_file(msg.user, i.file, msg.offline)
 
             elif i.__class__ is slskmessages.GetSharedFileList:
                 self.userbrowse.show_connection_error(msg.user)
@@ -457,13 +455,17 @@ class NicotineCore:
             elif i.__class__ is slskmessages.UserInfoRequest:
                 self.userinfo.show_connection_error(msg.user)
 
-    def message_progress(self, msg):
+    def peer_message_progress(self, msg):
 
         if msg.msg_type is slskmessages.SharedFileList:
-            self.userbrowse.message_progress(msg)
+            self.userbrowse.peer_message_progress(msg)
 
         elif msg.msg_type is slskmessages.UserInfoReply:
-            self.userinfo.message_progress(msg)
+            self.userinfo.peer_message_progress(msg)
+
+    def peer_connection_closed(self, msg):
+        self.userbrowse.peer_connection_closed(msg)
+        self.userinfo.peer_connection_closed(msg)
 
     def server_timeout(self, _msg):
         if not config.need_config():
@@ -471,7 +473,7 @@ class NicotineCore:
 
     def server_disconnect(self, msg=None):
 
-        self.logged_in = False
+        self.user_status = UserStatus.OFFLINE
 
         # Clean up connections
         self.user_statuses.clear()
@@ -479,6 +481,7 @@ class NicotineCore:
 
         self.pluginhandler.server_disconnect_notification(msg.manual_disconnect if msg else True)
 
+        self.shares.server_disconnect()
         self.transfers.server_disconnect()
         self.search.server_disconnect()
         self.userlist.server_disconnect()
@@ -498,7 +501,7 @@ class NicotineCore:
         """ Server code: 1 """
 
         if msg.success:
-            self.logged_in = True
+            self.user_status = UserStatus.ONLINE
             self.login_username = msg.username
 
             self.set_away_mode(config.sections["server"]["away"])
@@ -537,17 +540,14 @@ class NicotineCore:
 
         user = msg.user
 
-        # User seems to be offline, don't update IP
-        if msg.ip_address != "0.0.0.0":
+        # If the IP address changed, make sure our IP block/ignore list reflects this
+        self.network_filter.update_saved_user_ip_filters(user)
 
-            # If the IP address changed, make sure our IP block/ignore list reflects this
-            self.network_filter.update_saved_user_ip_filters(user)
+        if self.network_filter.block_unblock_user_ip_callback(user):
+            return
 
-            if self.network_filter.block_unblock_user_ip_callback(user):
-                return
-
-            if self.network_filter.ignore_unignore_user_ip_callback(user):
-                return
+        if self.network_filter.ignore_unignore_user_ip_callback(user):
+            return
 
         country_code = self.geoip.get_country_code(msg.ip_address)
 
