@@ -19,22 +19,24 @@
 import threading
 import socket
 
+from pynicotine.config import config
 from pynicotine.logfacility import log
 from pynicotine.utils import http_request
 
 
 MULTICAST_HOST = "239.255.255.250"
 MULTICAST_PORT = 1900
-RESPONSE_TIME_SECONDS = 2
+MULTICAST_TTL = 2  # Should default to 2 according to UPnP specification
+MX_RESPONSE_DELAY = 1  # At least 1 second is sufficient according to UPnP specification
+HTTP_REQUEST_TIMEOUT = 5
 
 
-class Router:
-    def __init__(self, wan_ip_type, url_scheme, base_url, root_url, service_type, control_url):
-        self.search_target = wan_ip_type
+class Service:
+    def __init__(self, service_type, url_scheme, base_url, root_url, control_url):
+        self.service_type = service_type
         self.url_scheme = url_scheme
         self.base_url = base_url
         self.root_url = root_url
-        self.service_type = service_type
         self.control_url = control_url
 
 
@@ -63,7 +65,7 @@ class SSDPRequest:
             "HOST": "%s:%s" % (MULTICAST_HOST, MULTICAST_PORT),
             "ST": search_target,
             "MAN": '"ssdp:discover"',
-            "MX": str(RESPONSE_TIME_SECONDS)
+            "MX": str(MX_RESPONSE_DELAY)
         }
 
     def sendto(self, sock, addr):
@@ -86,7 +88,7 @@ class SSDPRequest:
 class SSDP:
 
     @staticmethod
-    def get_router_control_url(url_scheme, base_url, root_url):
+    def get_service_control_url(url_scheme, base_url, root_url):
 
         service_type = None
         control_url = None
@@ -94,7 +96,7 @@ class SSDP:
         try:
             from xml.etree import ElementTree
 
-            response = http_request(url_scheme, base_url, root_url, timeout=2)
+            response = http_request(url_scheme, base_url, root_url, timeout=HTTP_REQUEST_TIMEOUT)
             log.add_debug("UPnP: Device description response from %s://%s%s: %s",
                           (url_scheme, base_url, root_url, response.encode('utf-8')))
 
@@ -119,7 +121,7 @@ class SSDP:
         return service_type, control_url
 
     @staticmethod
-    def add_router(routers, ssdp_response):
+    def add_service(services, locations, ssdp_response):
 
         from urllib.parse import urlsplit
         response_headers = {k.upper(): v for k, v in ssdp_response.headers}
@@ -130,8 +132,15 @@ class SSDP:
             log.add_debug("UPnP: M-SEARCH response did not contain a LOCATION header: %s", ssdp_response.headers)
             return
 
-        url_parts = urlsplit(response_headers["LOCATION"])
-        service_type, control_url = SSDP.get_router_control_url(url_parts.scheme, url_parts.netloc, url_parts.path)
+        location = response_headers["LOCATION"]
+
+        if location in locations:
+            log.add_debug("UPnP: Device location was previously processed, ignoring")
+            return
+
+        locations.add(location)
+        url_parts = urlsplit(location)
+        service_type, control_url = SSDP.get_service_control_url(url_parts.scheme, url_parts.netloc, url_parts.path)
 
         if service_type is None or control_url is None:
             log.add_debug("UPnP: No router with UPnP enabled in device search response, ignoring")
@@ -139,23 +148,26 @@ class SSDP:
 
         log.add_debug("UPnP: Device details: service_type '%s'; control_url '%s'", (service_type, control_url))
 
-        routers.append(
-            Router(wan_ip_type=response_headers['ST'], url_scheme=url_parts.scheme,
-                   base_url=url_parts.netloc, root_url=url_parts.path,
-                   service_type=service_type, control_url=control_url))
+        if service_type in services:
+            log.add_debug("UPnP: Service was previously added, ignoring")
+            return
 
-        log.add_debug("UPnP: Added device to list")
+        services[service_type] = Service(service_type=service_type, url_scheme=url_parts.scheme,
+                                         base_url=url_parts.netloc, root_url=url_parts.path, control_url=control_url)
+
+        log.add_debug("UPnP: Added service to list")
 
     @staticmethod
-    def get_routers(private_ip):
+    def get_services(private_ip):
 
-        log.add_debug("UPnP: Discovering... delay=%s seconds", RESPONSE_TIME_SECONDS)
+        log.add_debug("UPnP: Discovering... delay=%s seconds", MX_RESPONSE_DELAY)
 
         # Create a UDP socket and set its timeout
         with socket.socket(family=socket.AF_INET, type=socket.SOCK_DGRAM, proto=socket.IPPROTO_UDP) as sock:
-            sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, RESPONSE_TIME_SECONDS)
+            sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_IF, socket.inet_aton(private_ip))
+            sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, MULTICAST_TTL)
             sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
-            sock.settimeout(RESPONSE_TIME_SECONDS)
+            sock.settimeout(MX_RESPONSE_DELAY + 0.1)  # Larger timeout in case data arrives at the last moment
             sock.bind((private_ip, 0))
 
             # Protocol 1
@@ -182,34 +194,33 @@ class SSDP:
             wan_igd2.sendto(sock, (MULTICAST_HOST, MULTICAST_PORT))
             log.add_debug("UPnP: Sent M-SEARCH IGD request 2")
 
-            routers = []
+            locations = set()
+            services = {}
 
             while True:
                 try:
-                    message = sock.recv(4096)
-                    SSDP.add_router(routers, SSDPResponse(message.decode('utf-8')))
+                    message = sock.recv(65507)  # Maximum size of UDP message
+                    SSDP.add_service(services, locations, SSDPResponse(message.decode('utf-8')))
 
-                except socket.error:
+                except socket.timeout:
                     break
 
-            log.add_debug("UPnP: %s device(s) detected", str(len(routers)))
+            log.add_debug("UPnP: %s service(s) detected", str(len(services)))
 
-        return routers
+        return services
 
 
 class UPnP:
     """ Class that handles UPnP Port Mapping """
 
-    def __init__(self, core, config):
+    def __init__(self, port, local_ip_address):
 
-        self.core = core
-        self.config = config
+        self.port = port
+        self.local_ip_address = local_ip_address
         self.timer = None
 
-        self.add_port_mapping()
-
     @staticmethod
-    def _request_port_mapping(router, protocol, public_port, private_ip, private_port,
+    def _request_port_mapping(service, protocol, public_port, private_ip, private_port,
                               mapping_description, lease_duration):
         """
         Function that adds a port mapping to the router.
@@ -218,14 +229,14 @@ class UPnP:
 
         from xml.etree import ElementTree
 
-        url = '%s%s' % (router.base_url, router.control_url)
+        url = '%s%s' % (service.base_url, service.control_url)
         log.add_debug("UPnP: Adding port mapping (%s %s/%s, %s) at url '%s'",
-                      (private_ip, private_port, protocol, router.search_target, url))
+                      (private_ip, private_port, protocol, service.service_type, url))
 
         headers = {
-            "Host": router.base_url,
+            "Host": service.base_url,
             "Content-Type": "text/xml; charset=utf-8",
-            "SOAPACTION": '"%s#AddPortMapping"' % router.service_type
+            "SOAPACTION": '"%s#AddPortMapping"' % service.service_type
         }
 
         body = (
@@ -245,7 +256,7 @@ class UPnP:
              + '</u:AddPortMapping>'
              + '</s:Body>'
              + '</s:Envelope>\r\n') %
-            (router.service_type, public_port, protocol, private_port, private_ip,
+            (service.service_type, public_port, protocol, private_port, private_ip,
              mapping_description, lease_duration)
         ).encode('utf-8')
 
@@ -253,8 +264,8 @@ class UPnP:
         log.add_debug("UPnP: Add port mapping request contents: %s", body)
 
         response = http_request(
-            router.url_scheme, router.base_url, router.control_url,
-            request_type="POST", body=body, headers=headers)
+            service.url_scheme, service.base_url, service.control_url,
+            request_type="POST", body=body, headers=headers, timeout=HTTP_REQUEST_TIMEOUT)
 
         xml = ElementTree.fromstring(response)
 
@@ -266,53 +277,23 @@ class UPnP:
         error_code = xml.findtext(".//{urn:schemas-upnp-org:control-1-0}errorCode")
         error_description = xml.findtext(".//{urn:schemas-upnp-org:control-1-0}errorDescription")
 
-        if error_code or error_description:
-            raise Exception(_("Error code %(code)s: %(description)s") %
-                            {"code": error_code, "description": error_description})
+        return error_code, error_description
 
     @staticmethod
-    def find_local_ip_address():
+    def find_service(private_ip):
 
-        # Create a UDP socket
-        with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as local_socket:
+        services = SSDP.get_services(private_ip)
+        service = services.get("urn:schemas-upnp-org:service:WANIPConnection:2")
 
-            # Send a broadcast packet on a local address (doesn't need to be reachable,
-            # but MacOS requires port to be non-zero)
-            local_socket.connect(("10.255.255.255", 1))
+        if not service:
+            service = services.get("urn:schemas-upnp-org:service:WANIPConnection:1")
 
-            # This returns the "primary" IP on the local box, even if that IP is a NAT/private/internal IP
-            ip_address = local_socket.getsockname()[0]
+        if not service:
+            service = services.get("urn:schemas-upnp-org:service:WANPPPConnection:1")
 
-        return ip_address
+        return service
 
-    @staticmethod
-    def find_router(private_ip):
-
-        routers = SSDP.get_routers(private_ip)
-        router = next((r for r in routers if r.search_target == "urn:schemas-upnp-org:service:WANIPConnection:2"), None)
-
-        if not router:
-            router = next(
-                (r for r in routers if r.search_target == "urn:schemas-upnp-org:service:WANIPConnection:1"), None)
-
-        if not router:
-            router = next(
-                (r for r in routers if r.search_target == "urn:schemas-upnp-org:service:WANPPPConnection:1"), None)
-
-        if not router:
-            router = next(
-                (r for r in routers if r.search_target == "urn:schemas-upnp-org:device:InternetGatewayDevice:2"), None)
-
-        if not router:
-            router = next(
-                (r for r in routers if r.search_target == "urn:schemas-upnp-org:device:InternetGatewayDevice:1"), None)
-
-        if not router:
-            router = next((r for r in routers), None)
-
-        return router
-
-    def _update_port_mapping(self, listening_port):
+    def _update_port_mapping(self, lease_duration=86400):
         """
         This function supports creating a Port Mapping via the UPnP
         IGDv1 and IGDv2 protocol.
@@ -326,41 +307,42 @@ class UPnP:
         try:
             log.add_debug("UPnP: Creating Port Mapping rule...")
 
-            # Find local IP address
-            local_ip_address = self.find_local_ip_address()
-
             # Find router
-            router = self.find_router(local_ip_address)
+            service = self.find_service(self.local_ip_address)
 
-            if not router:
+            if not service:
                 raise RuntimeError(_("UPnP is not available on this network"))
 
             # Perform the port mapping
             log.add_debug("UPnP: Trying to redirect external WAN port %s TCP => %s port %s TCP", (
-                listening_port,
-                local_ip_address,
-                listening_port
+                self.port,
+                self.local_ip_address,
+                self.port
             ))
 
-            try:
-                self._request_port_mapping(
-                    router=router,
-                    protocol="TCP",
-                    public_port=listening_port,
-                    private_ip=local_ip_address,
-                    private_port=listening_port,
-                    mapping_description="NicotinePlus",
-                    lease_duration=86400  # Expires in 24 hours
-                )
+            error_code, error_description = self._request_port_mapping(
+                service=service,
+                protocol="TCP",
+                public_port=self.port,
+                private_ip=self.local_ip_address,
+                private_port=self.port,
+                mapping_description="NicotinePlus",
+                lease_duration=lease_duration
+            )
 
-            except Exception as error:
-                raise RuntimeError(
-                    _("Failed to map the external WAN port: %(error)s") % {"error": error}) from error
+            if error_code == "725" and lease_duration > 0:
+                log.add_debug("UPnP: Router requested permanent lease duration")
+                self._update_port_mapping(lease_duration=0)
+                return
+
+            if error_code or error_description:
+                raise Exception(_("Error code %(code)s: %(description)s") %
+                                {"code": error_code, "description": error_description})
 
         except Exception as error:
             from traceback import format_exc
             log.add(_("UPnP: Failed to forward external port %(external_port)s: %(error)s"), {
-                "external_port": listening_port,
+                "external_port": self.port,
                 "error": error
             })
             log.add_debug(format_exc())
@@ -368,35 +350,35 @@ class UPnP:
 
         log.add(_("UPnP: External port %(external_port)s successfully forwarded to local "
                   "IP address %(ip_address)s port %(local_port)s"), {
-            "external_port": listening_port,
-            "ip_address": local_ip_address,
-            "local_port": listening_port
+            "external_port": self.port,
+            "ip_address": self.local_ip_address,
+            "local_port": self.port
         })
 
-    def add_port_mapping(self):
+    def add_port_mapping(self, blocking=False):
 
         # Test if we want to do a port mapping
-        if not self.config.sections["server"]["upnp"]:
+        if not config.sections["server"]["upnp"]:
             return
 
         # Do the port mapping
-        thread = threading.Thread(target=self._add_port_mapping)
-        thread.name = "UPnPAddPortmapping"
-        thread.daemon = True
-        thread.start()
+        if blocking:
+            self._update_port_mapping()
+        else:
+            thread = threading.Thread(target=self._update_port_mapping)
+            thread.name = "UPnPAddPortmapping"
+            thread.daemon = True
+            thread.start()
 
         # Repeat
         self._start_timer()
 
-    def _add_port_mapping(self):
-        self._update_port_mapping(self.core.protothread.listenport)
-
     def _start_timer(self):
         """ Port mapping entries last 24 hours, we need to regularly renew them.
         The default interval is 4 hours. """
 
         self.cancel_timer()
-        upnp_interval = self.config.sections["server"]["upnp_interval"]
+        upnp_interval = config.sections["server"]["upnp_interval"]
 
         if upnp_interval < 1:
             return
